@@ -16,9 +16,9 @@
 #include <debug_cuda.hpp>
 #include "config.hpp"
 #include <memory.hpp>
-#include <boost/scoped_ptr.hpp>
+#include <boost/scoped_array.hpp>
 
-using boost::scoped_ptr;
+using boost::scoped_array;
 
 namespace cuda
 {
@@ -486,8 +486,8 @@ namespace kernel
             tlptr = memAlloc<uint>(tmp_elements);
             ireduce_first_launcher<T, op, true>(tmp, tlptr, in, NULL, blocks_x, blocks_y, threads_x);
 
-            scoped_ptr<T>       h_ptr(new T[tmp_elements]);
-            scoped_ptr<uint>    h_lptr(new uint[tmp_elements]);
+            scoped_array<T>       h_ptr(new T[tmp_elements]);
+            scoped_array<uint>    h_lptr(new uint[tmp_elements]);
             T*      h_ptr_raw = h_ptr.get();
             uint*   h_lptr_raw = h_lptr.get();
 
@@ -520,7 +520,7 @@ namespace kernel
             return Op.m_val;
         } else {
 
-            scoped_ptr<T> h_ptr(new T[in_elements]);
+            scoped_array<T> h_ptr(new T[in_elements]);
             T* h_ptr_raw = h_ptr.get();
             CUDA_CHECK(cudaMemcpyAsync(h_ptr_raw, in.ptr, in_elements * sizeof(T),
                        cudaMemcpyDeviceToHost, cuda::getStream(cuda::getActiveDeviceId())));
